DECLARE FUNCTION Fixpath$ (path$)
DECLARE FUNCTION ExtractPath$ (n$)
DECLARE FUNCTION ExtractName$ (n$)
DECLARE FUNCTION GetCurPath$ ()
DECLARE SUB ChDrive (drive$)
DECLARE FUNCTION Dir$ (filespec$, attrib AS INTEGER)
DECLARE FUNCTION ValidDrive% (d$)
DECLARE FUNCTION SplitString% (s$, sep$, array$(), max%)
DECLARE SUB QuickSort (sortarray$(), Lower%, Upper%)
'Define the type needed for INTERRUPT calls
TYPE RegType
	 ax    AS INTEGER
	 bx    AS INTEGER
	 cx    AS INTEGER
	 dx    AS INTEGER
	 bp    AS INTEGER
	 si    AS INTEGER
	 di    AS INTEGER
	 flags AS INTEGER
END TYPE
' Define the type needed for INTERRUPTX calls
TYPE RegTypeX
	 ax    AS INTEGER
	 bx    AS INTEGER
	 cx    AS INTEGER
	 dx    AS INTEGER
	 bp    AS INTEGER
	 si    AS INTEGER
	 di    AS INTEGER
	 flags AS INTEGER
	 ds    AS INTEGER
	 es    AS INTEGER
END TYPE
' Generate a software interrupt, loading all but the segment registers
DECLARE SUB INTERRUPT (intnum AS INTEGER, inreg AS RegType, outreg AS RegType)
' Generate a software interrupt, loading all registers
DECLARE SUB INTERRUPTX (intnum AS INTEGER, inreg AS RegTypeX, outreg AS RegTypeX)

	SYSTEM
ChDriveErr:
	ErrorNo = 1
	RESUME NEXT

DEFINT A-Z
SUB ChDrive (drive$)
	DIM regs AS RegType
   
	ON ERROR GOTO ChDriveErr

	'This is so that we will get a qb error if the a: drive is not ready
	CHDIR LEFT$(drive$, 1) + ":"
	ON ERROR GOTO 0
	IF ErrorNo = 1 THEN
		ErrorNo = 0
		EXIT SUB
	END IF

	drive$ = UCASE$(drive$)
	driven = ASC(LEFT$(drive$, 1)) - 65
	IF driven < 0 OR driven > 25 THEN EXIT SUB
	regs.ax = &HE00
	regs.dx = driven
	INTERRUPT &H21, regs, regs
	EXIT SUB
END SUB

FUNCTION Dir$ (filespec$, attrib AS INTEGER) STATIC
 
	CONST DOS = &H21
	CONST SetDTA = &H1A00, FindFirst = &H4E00, FindNext = &H4F00
   
	DIM DTA AS STRING * 128, regs AS RegTypeX
	Null$ = CHR$(0)
 
ReTry:
 
	'-----  Set up our own DTA so we don't destroy COMMAND$
	regs.ax = SetDTA                    'Set DTA function
	regs.dx = VARPTR(DTA)               'DS:DX points to our DTA
	regs.ds = -1                        'Use current value for DS
	INTERRUPTX DOS, regs, regs          'Do the interrupt
 
	'-----  Check to see if this is First or Next
	IF filespec$ <> "" THEN            'FileSpec$ isn't null, so FindFirst
		FileSpecZ$ = filespec$ + Null$   'Make FileSpec$ into an ASCIIZ
											'string
		regs.ax = FindFirst              'Perform a FindFirst
		regs.cx = attrib
		regs.dx = SADD(FileSpecZ$)       'DS:DX points to ASCIIZ file
		regs.ds = -1                     'Use current DS
	ELSE                                 'We have a null FileSpec$,
		regs.ax = FindNext               'so FindNext
	END IF
 
	INTERRUPTX DOS, regs, regs          'Do the interrupt
 
	 '-----  Return file name or null
	 IF regs.flags AND 1 THEN            'No files found
		 Dir$ = ""                        'Return null string
		 EXIT FUNCTION
	 ELSE
		'* check if its . or .., if so, can't accept it.
		IF ASC(MID$(DTA, 22, 1)) <> attrib AND attrib <> 0 THEN
			'attribs don't match, so skip it
			 filespec$ = ""
			 GOTO ReTry
		END IF
		IF MID$(DTA, 31, 1) = "." THEN   '*If its . or ..
			filespec$ = ""
			GOTO ReTry
		END IF
	 END IF
	 Null = INSTR(31, DTA, Null$)     'Get the filename found
	 Dir$ = MID$(DTA, 31, Null - 31)  'It's an ASCIIZ string starting
										 'at offset 30 of the DTA
END FUNCTION

FUNCTION ExtractName$ (n$)
	t = 0
	FOR i = LEN(n$) TO 1 STEP -1
		IF MID$(n$, i, 1) = "\" THEN
			t = i
			EXIT FOR
		END IF
	NEXT
	IF t <> LEN(n$) THEN
		p$ = RIGHT$(n$, LEN(n$) - t)
	ELSE
		p$ = ""
	END IF
	ExtractName$ = p$
END FUNCTION

FUNCTION ExtractPath$ (n$)
	t = 0
	FOR i = LEN(n$) TO 1 STEP -1
		IF MID$(n$, i, 1) = "\" THEN
			t = i
			EXIT FOR
		END IF
	NEXT
	IF t > 0 THEN
		p$ = LEFT$(n$, t)
	ELSE
		p$ = ""
	END IF
	ExtractPath$ = p$
END FUNCTION

FUNCTION Fixpath$ (path$)
	p$ = path$
	IF p$ = "" THEN
		Fixpath$ = p$
		EXIT FUNCTION
	END IF
	p$ = LTRIM$(RTRIM$(p$))
	IF RIGHT$(p$, 1) <> "\" THEN
		p$ = p$ + "\"
	END IF
	Fixpath$ = p$
END FUNCTION

FUNCTION GetCurPath$
	DIM regs AS RegType
	DIM regsx AS RegTypeX
	buffer$ = STRING$(256, CHR$(0))

	'get path
	regsx.ax = &H4700
	regsx.dx = 0
	regsx.ds = VARSEG(buffer$)
	regsx.si = SADD(buffer$) + 3
	INTERRUPTX &H21, regsx, regsx

	'get drive
	regs.ax = &H1900
	INTERRUPT &H21, regs, regs
	MID$(buffer$, 1, 1) = CHR$(regs.ax MOD 256 + 65)
	MID$(buffer$, 2, 2) = ":\"
	buffer$ = UCASE$(buffer$)
	GetCurPath$ = LEFT$(buffer$, INSTR(buffer$, CHR$(0)) - 1)
END FUNCTION

SUB QuickSort (sortarray$(), Lower%, Upper%)
	'QuickSort iterative (rather than recursive) by Cornel Huth

	DIM lStack(1 TO 128) AS INTEGER 'low stack
	DIM hStack(1 TO 128) AS INTEGER 'high stack
	DIM sp AS INTEGER 'stack pointer
	DIM i AS INTEGER, j AS INTEGER
	DIM Compare AS STRING
   
	IF Lower% > Upper% THEN EXIT SUB

	sp = 1
	lStack(sp) = Lower%
	hStack(sp) = Upper%
	sp = sp + 1
  
	DO
		sp = sp - 1
		Lo = lStack(sp)
		Hi = hStack(sp)
		DO
			i = Lo
			j = Hi
			Mid = (Lo + Hi) \ 2
			Compare = sortarray$(Mid)
			DO
				DO WHILE sortarray$(i) < Compare
					i = i + 1
				LOOP
				DO WHILE sortarray$(j) > Compare
					j = j - 1
				LOOP
				IF i <= j THEN
					SWAP sortarray$(i), sortarray$(j)
					i = i + 1
					j = j - 1
				END IF
			LOOP WHILE i <= j
			IF j - Lo < Hi - i THEN
				IF i < Hi THEN
					  lStack(sp) = i
					  hStack(sp) = Hi
					  sp = sp + 1
				END IF
				Hi = j
			ELSE
				IF Lo < j THEN
					  lStack(sp) = Lo
					  hStack(sp) = j
					  sp = sp + 1
				END IF
				Lo = i
			END IF
		LOOP WHILE Lo < Hi
	LOOP WHILE sp <> 1
END SUB

FUNCTION SplitString (s$, sep$, array$(), max)
	ind = 0
	last = 1
	s2$ = s$ + LEFT$(sep$, 1)
	FOR i = 1 TO LEN(s2$)
		c$ = MID$(s2$, i, 1)
		IF INSTR(sep$, c$) <> 0 THEN
			IF last <> i THEN
				array$(ind) = MID$(s2$, last, i - last)
				ind = ind + 1
				IF ind = max THEN EXIT FOR
			END IF
			last = i + 1
		END IF
	NEXT
	SplitString = ind
END FUNCTION

FUNCTION ValidDrive (d$)
	DIM FCBBuf AS STRING * 40
	DIM drive AS STRING * 3
	'NumDrives = 0
	'EXIT FUNCTION
   
	drive = UCASE$(LEFT$(d$, 1)) + ":" + CHR$(0)
	DIM regs AS RegTypeX
	regs.ax = &H2906
	regs.ds = VARSEG(drive)
	regs.si = VARPTR(drive)
	regs.es = VARSEG(FCBBuf)
	regs.di = VARPTR(FCBBuf)
	INTERRUPTX &H21, regs, regs
	IF (regs.ax MOD 256) = &HFF THEN
		ValidDrive = 0
	ELSE
		ValidDrive = 1
	END IF
END FUNCTION

